@@ -1,6 +1,7 @@
 use synstructure::{quote, BindStyle};
 use proc_macro2::{TokenStream, Ident, Span};
 use quote::ToTokens;
+use syn::GenericParam;
 
 fn reroot_derive(mut s: synstructure::Structure) -> proc_macro2::TokenStream {
     s.bind_with(|_| {
@@ -17,8 +18,21 @@ fn reroot_derive(mut s: synstructure::Structure) -> proc_macro2::TokenStream {
     let ident = sname.ident;
     let gens = sname.generics;
 
+    let mut gen_fixed = TokenStream::new();
+
+    for gen in gens.params.iter() {
+        match gen {
+            GenericParam::Type(tp) => quote! { #tp, },
+            GenericParam::Lifetime(ltd) => quote! { #ltd, },
+            GenericParam::Const(cp) => {
+                let ident = &cp.ident;
+                quote! { #ident, }
+            }
+        }.to_tokens(&mut gen_fixed);
+    }
+
     quote!{
-        #ident #gens
+        #ident < #gen_fixed >
     }.to_tokens(&mut ret_val);
 
     let mut body = TokenStream::new();
